
### Custom Keyboard Maestro Plug-in
#### NAME
- Copy Search Results from Safari to Markdown

#### VERSION
- 0.1

#### SYNOPSIS
- Finds a specific subset of links or other elements (in the current Safari tab), and
- returns their content as a Markdown (or simple plain text) list.

#### REQUIREMENTS
- Yosemite
	(The core script `harvestList.sh` is mainly written in Javascript for Applications)
	
#### OPTIONS
##### Mode
- **Use default settings for common websites**
	- Search Engines
		- Google (web|scholar|book), DuckDuckGo, Yahoo, Bing
	- Databases
		- JSTOR, PubMed, Library of Congress, WorldCat, British Library, London Library, MLA
	- Book sellers
		- Abebooks, AddAll, Amazon
	- Lists of posts in sample forums
		- Keyboard Maestro, FoldingText, StackOverflow
	- Other
		- https://kindle.amazon.com/your_highlights/
		
- **Use custom settings for a particular website**
	- Use the custom Xpath, list title, and item format settings in the fields below.
	
- **Show this help file**
	- Open in Preview as a pdf

##### Custom XPath
- Limit the links listed to those on a particular path, for example to harvest just the main result links on a search page, while ignoring secondary links like `Prev`, `Next`, `Home` etc

- Uses a simple [XPath](http://en.wikipedia.org/wiki/XPath) search pattern,
	- which is normally **automatically** retrieved by the plugin for a recognised website,
	- but can be a **custom** XPath expression given by the user:
		- e.g. only `<a>` links within `h3` headings:
			- *xpath:* `//h3/a`
		- or only `<a>` links within `<p>` paragraphs of class `title`:
			- *xpath:* `//p[@class='title']/a`
				
- where
	- `//` means *'in any descendant of the root node'*, or effectively *'any where in the HTML page'*
	- simple unquoted strings are HTML element names like `h2`, `p`, `a`
	- attribute names are preceded by `@`,

- and
	1. the left to right segmentation by `/` gives a nesting path, as in an OS X file path, and
	2.  square brackets can enclose either numeric indices (*fourth para* = `p[4]`), or `[@attrib='value']` filtering conditions, like "only if its class is **title**":  `[@class='title']`


- To see the path of the elements required, ctrl-click one of them in Safari, and choose `Inspect element` from the menu.

##### Custom list title
- An optional string to use as a heading for the list
- No heading will be used, in custom mode, if this is left blank

##### Custom format for each x
- The format of items in the list is given using:
	- literal strings in 'single quotes',
	- the javascript `+` concatenation operator, and
	- references to the matched link (or other element) as `x`, 
	- and to its properties as `x.href`, `x.text` etc.
	
- e.g. if we are matching `<a>` links, and want to derive bulleted markdown links from their `href` attributes and enclosed text:
	       `'- [' + x.text + '](' + x.href + ')'`

#### INSTALLATION
- Drag the .zip file onto the Keyboard Maestro icon in the OS X toolbar.
- (if updating a previous version of the action, first manually remove the previous copy from the custom actions folder)
	- `~/Library/Application Support/Keyboard Maestro/Keyboard Maestro Actions`

---

#### EXAMPLES
- Web sites which are automatically recognised by the plugin, and
- the (xpath + list item) settings which are used for them.


##### Google search results
[https://www.google.com/](https://www.google.com/)
<font color="gray">**XPath**:</font> `//h3[@class='r']/a`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](' + x.href + ')'`</font>

##### Google Scholar search results
[https://scholar.google.com/](https://scholar.google.com/)
<font color="gray">**XPath**:</font> `//h3[@class='gs_rt']/a`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](' + x.href + ')'`</font>

##### Google Books search results
[https://books.google.com/](https://books.google.com/)
<font color="gray">**XPath**:</font> `//h3[@class='r']/a`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](' + x.href + ')'`</font>

##### DuckDuckGo search results
[https://duckduckgo.com/](https://duckduckgo.com/)
<font color="gray">**XPath**:</font> `//h2[@class='result__title']/a`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](' + x.href + ')'`</font>

##### Yahoo search results
[https://us.search.yahoo.com/](https://us.search.yahoo.com/)
<font color="gray">**XPath**:</font> `//h3[@class='title']/a[@class=' td-u']`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](' + x.href + ')'`</font>

##### Bing search results
[https://www.bing.com/](https://www.bing.com/)
<font color="gray">**XPath**:</font> `//li/h2/a`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](' + x.href + ')'`</font>

##### JSTOR search results
[http://www.jstor.org/](http://www.jstor.org/)
<font color="gray">**XPath**:</font> `//div[@class='title']/a`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](http://www.jstor.org' + x.href + ')'`</font>

##### PubMed search results
[http://www.ncbi.nlm.nih.gov/](http://www.ncbi.nlm.nih.gov/)
<font color="gray">**XPath**:</font> `//p[@class='title']/a`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](http://www.ncbi.nlm.nih.gov' + x.href + ')'`</font>

##### WorldCat search results
[http://www.worldcat.org/](http://www.worldcat.org/)
<font color="gray">**XPath**:</font> `//div[@class='name']/a`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](http://www.worldcat.org' + x.href + ')'`</font>

##### British Library Catalogue Search
[http://catalogue.bl.uk/](http://catalogue.bl.uk/)
<font color="gray">**XPath**:</font> `//h2[@class='EXLResultTitle']/a`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](http://catalogue.bl.uk/primo_library/libweb/action/' + x.href + ')'`</font>

##### Library of Congress Catalogue Search
[http://www.loc.gov/](http://www.loc.gov/)
<font color="gray">**XPath**:</font> `//h2/a`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](http:' + x.href + ')'`</font>

##### London Library Catalogue Search
[http://pmt-eu.hosted.exlibrisgroup.com/](http://pmt-eu.hosted.exlibrisgroup.com/)
<font color="gray">**XPath**:</font> `//h2[@class='EXLResultTitle']/a`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](http://pmt-eu.hosted.exlibrisgroup.com/primo_library/libweb/action/' + x.href + ')'`</font>

##### MLA search results
[http://www.mla.org/](http://www.mla.org/)
<font color="gray">**XPath**:</font> `//a[@class='gs-title']`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](' + x.href + ')'`</font>

##### AbeBooks search results
[http://www.abebooks.com/](http://www.abebooks.com/)
<font color="gray">**XPath**:</font> `//h2[@itemprop='offers']/a`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.title + '](http://www.abebooks.com' + x.href + ')'`</font>

##### AddAll book search results
[http://www.addall.com/](http://www.addall.com/)
<font color="gray">**XPath**:</font> `//dt/b/font/a`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](http://www.addall.com' + x.href + ')'`</font>

##### StackOverflow search results
[http://stackoverflow.com/](http://stackoverflow.com/)
<font color="gray">**XPath**:</font> `//div[@class='result-link']/span/a`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](http://www.stackoverflow.com' + x.href + ')'`</font>

##### FoldingText forum posts
[http://support.foldingtext.com/](http://support.foldingtext.com/)
<font color="gray">**XPath**:</font> `//td[@class='main-link clearfix']/a`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](http://support.foldingtext.com' + x.href + ')'`</font>

##### Keyboard Maestro forum posts
[http://forum.keyboardmaestro.com/](http://forum.keyboardmaestro.com/)
<font color="gray">**XPath**:</font> `//td[@class='main-link clearfix']/a`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](http://forum.keyboardmaestro.com' + x.href + ')'`</font>

##### Kindle books with highlights
[https://kindle.amazon.com/your_highlights/](https://kindle.amazon.com/your_highlights/)
<font color="gray">**XPath**:</font> `//span[@class='title']/a`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.text + '](https://kindle.amazon.com' + x.href + ')'`</font>

##### Amazon search results
[http://www.amazon.com/](http://www.amazon.com/)
<font color="gray">**XPath**:</font> `//a[@class='a-link-normal s-access-detail-page  a-text-normal']`
<font color="red">**λx**:</font> <font color="gray">`'- [' + x.title + '](' + x.href + ')'`</font>


#### CONTACT
- (For example to suggest the addition of any further search engines or databases for **automatic** mode)
- Rob Trew - Twitter @ComplexPoint